/*******************************************************************************
 * Copyright 2017 Frank Mnster
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrcodehandler.bcdsepa;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Locale;

import de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerException;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerRegsistry;
import de.frankmuenster.jameica.qrscanner.Settings;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Implementiert das Interface {@link de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler} mit einem Handler fr BCD
 * SEPA berweisungen.
 *
 * @author Frank Mnster
 */
public class BcdSepa_001_SCT implements IQrCodeHandler {

    static {
        QrCodeHandlerRegsistry.register(new BcdSepa_001_SCT());
    }

    /**
     * Standardkonstruktor
     */
    public BcdSepa_001_SCT() {
        // Absichtlich leer
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#handleQrCode(java.lang.String)
     */
    @Override
    public void handleQrCode(final String qrCode) throws QrCodeHandlerException {
        Logger.debug("BCD Code Version 001 - SCT");
        final String qrCodeLow = qrCode.toLowerCase(Locale.getDefault());
        if (!qrCodeLow.startsWith("bcd\n") && !qrCodeLow.startsWith("bcd\r\n")) {
            Logger.error("Falscher Qr-Code. Beginnt nicht mit: 'bcd[Zeilenumbruch]'");
            Logger.debug("Qr-Code: " + qrCode);
            throw new QrCodeHandlerException("Qr-Code not valid");
        }
        // Ist Hibiscus installiert?
        if (Settings.isHibiscusInstalled()) {
            final String[] qrCodeFields = qrCode.split("\\r?\\n");
            final ArrayList<String> qrFields = new ArrayList<>(Arrays.asList(qrCodeFields));
            if (qrFields.size() < 7) { // NOPMD
                throw new QrCodeHandlerException("Qr-Code ist zu kurz. Muss mindestens 7 Element endhalten!.");
            }
            // Auffuellen der List emit moeglich fehlenden Zeilen
            if (qrFields.size() < 12) { // NOPMD
                for (int i = qrFields.size(); i <= 11; i++) {
                    qrFields.add(""); //$NON-NLS-1$
                }
            }
            final BcdDataTransferObject bcdData = new BcdDataTransferObject();
            try {
                bcdData.setServiceKennung(qrFields.get(0));
                bcdData.setVersion(qrFields.get(1));
                bcdData.setKodierung(qrFields.get(2));
                bcdData.setFunktion(qrFields.get(3));
                bcdData.setBic(qrFields.get(4));
                bcdData.setEmpfaenger(qrFields.get(5));
                bcdData.setIban(qrFields.get(6));
                bcdData.setBetrag(qrFields.get(7));
                bcdData.setGeschaeftsCode(qrFields.get(8));
                bcdData.setReferenze(qrFields.get(9));
                bcdData.setVerwendungszweck(qrFields.get(10));
                bcdData.setBenutzerhinweis(qrFields.get(11));
                bcdData.oeffneNeueSEPA();
            } catch (final BcdDataTransferObjcetDefinitionException bcde) {
                Logger.error("BCD Datenobjekt-Fehler: " + bcde.getMessage());
                throw new QrCodeHandlerException(bcde.getMessage(), bcde);
            }
        } else {
            final String text = "Der QR-Code enthlt eine SEPA-berweisung, aber Hibiscus ist nicht installiert.\n"
                    + "Bitte installieren sie das Hibiscus-Plugin.";
            Logger.warn(text);
            try {
                Application.getCallback().notifyUser(text);
            } catch (final Exception e) { // NOPMD
                Logger.error("Info-Fenster konnte nicht geffnet werden. Fehler:\n" + e.getMessage());
            }
        }
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#canHandle(java.lang.String)
     */
    @Override
    public boolean canHandle(final String qrCode) {
        // Passt der QR-Code
        final String qrCodeLow = qrCode.toLowerCase(Locale.getDefault());
        return qrCodeLow.startsWith("bcd\n001") || qrCodeLow.startsWith("bcd\r\n001");
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#getHandlerName()
     */
    @Override
    public String getHandlerName() {
        return "BCD Vers. 001"; //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#isForBanking()
     */
    @Override
    public boolean isUeberweisung() {
        return true;
    }
}
