/*******************************************************************************
 * Copyright 2017 Frank Mnster
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrcodehandler.bezahlcode;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.rmi.RemoteException;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.SystemUtils;

import de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerException;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerRegsistry;
import de.frankmuenster.jameica.qrscanner.Settings;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.hbci.rmi.HibiscusAddress;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Implementiert das Interface {@link de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler} mit einem Hanlder fr
 * Bezahlcode Kontak_V2 (SEPA)
 *
 * @author munstfr
 * @version 1.0
 */
public class BezahlcodeContactV2Handler implements IQrCodeHandler {

    /**
     * Name des Handlers
     */
    private static final String HANDLER_NAME = "BezahlCode - Authority:contact and contact_v2";

    /**
     * Handler ID fr contact v1
     */
    private static final String HANDLER_ID_1 = "bank://contact?";

    /**
     * Handler ID fr contact v2
     */
    private static final String HANDLER_ID_2 = "bank://contact_v2?";
    /**
     * Handler registrieren
     */
    static {
        QrCodeHandlerRegsistry.register(new BezahlcodeContactV2Handler());
    }

    /**
     * Standardkonstruktor
     */
    public BezahlcodeContactV2Handler() {
        // Absichtlich leer
    }

    /**
     * Liest den QR-Code und erstellt den Kontakt.
     *
     * @param qrCode
     *            der QR-Code
     * @throws QrCodeHandlerException
     */
    @Override
    public void handleQrCode(final String qrCode) throws QrCodeHandlerException {
        if (!qrCode.toLowerCase(Locale.getDefault()).startsWith(BezahlcodeContactV2Handler.HANDLER_ID_1)
                && !qrCode.toLowerCase(Locale.getDefault()).startsWith(BezahlcodeContactV2Handler.HANDLER_ID_2)) {
            Logger.error("Falscher Qr-Code. Beginnt nicht mit: '" + BezahlcodeContactV2Handler.HANDLER_ID_1 + "' oder '"
                    + BezahlcodeContactV2Handler.HANDLER_ID_2 + "'");
            Logger.debug("Qr-Code: " + qrCode);
            throw new QrCodeHandlerException("Qr-Code not valid");
        }
        if (Settings.isHibiscusInstalled()) {
            try {
                final String qrCodeDecode = URLDecoder.decode(qrCode, "ISO-8859-1");
                final String qrCodeQuery = qrCodeDecode.substring(qrCodeDecode.indexOf('?') + 1);
                final Map<String, String> params = BezahlCodeCommenFunctions.getQueryMap(qrCodeQuery);
                final HibiscusAddress address = this.getAdress(params);
                if (address != null) {
                    // View neu zeichnen
                    GUI.startView(de.willuhn.jameica.hbci.gui.views.EmpfaengerNew.class, address);
                }
            } catch (final UnsupportedEncodingException e1) {
                throw new QrCodeHandlerException("QR-Code konnte nicht valide dekodiert werden.", e1);
            } catch (final RemoteException e) {
                throw new QrCodeHandlerException("Hibiscus Empfngersobject konnte nicht erstellt werden", e);
            }
        } else {
            final String text = "Der QR-Code enthlt eine berweisung, aber Hibiscus ist nicht installiert.\n"
                    + "Bitte installieren sie das Hibiscus-Plugin.";
            Logger.warn(text);
            try {
                Application.getCallback().notifyUser(text);
            } catch (final Exception e) { // NOPMD
                Logger.error("Info-Fenster konnte nicht geffnet werden. Fehler:\n" + e.getMessage());
            }
        }
    }

    /**
     * @param params
     * @return
     * @throws RemoteException
     */
    @SuppressWarnings({ "PMD.NPathComplexity", "PMD.ModifiedCyclomaticComplexity", "PMD.CyclomaticComplexity" })
    private HibiscusAddress getAdress(final Map<String, String> params) throws RemoteException {
        final HibiscusAddress address = (HibiscusAddress) de.willuhn.jameica.hbci.Settings.getDBService()
                .createObject(HibiscusAddress.class, null);
        // Name
        String value;
        if ((value = params.get("name")) != null) {
            if (value.equals(value.toUpperCase(Locale.getDefault()))) {
                try {
                    if (Application.getCallback().askUser(BezahlCodeCommenFunctions.QUESTION_TEXT,
                            new String[] { value })) {
                        value = BezahlCodeCommenFunctions.formatString(value);
                    }
                } catch (final Exception e) { // NOPMD
                    Logger.error("Benutzerabfrage konnte nicht geffnet werden. Fehler:\n" + e.getMessage());
                }
            }
            address.setName(value);
        }
        if ((value = params.get("account")) != null) {
            address.setKontonummer(value);
        }
        if ((value = params.get("bnc")) != null) {
            address.setBlz(value);
        }
        if ((value = params.get("iban")) != null) {
            address.setIban(value);
        }
        if ((value = params.get("bic")) != null) {
            address.setBic(value);
        }
        if ((value = params.get("reason")) == null) {
            // Test for reasonN
            final StringBuilder reasonN = new StringBuilder();
            if ((value = params.get("reason1")) != null) {
                reasonN.append(value);
            }
            if ((value = params.get("reason2")) != null) {
                reasonN.append(SystemUtils.LINE_SEPARATOR).append(value);
            }
            if ((value = params.get("reason3")) != null) {
                reasonN.append(SystemUtils.LINE_SEPARATOR).append(value);
            }
            if ((value = params.get("reason4")) != null) {
                reasonN.append(SystemUtils.LINE_SEPARATOR).append(value);
            }
            address.setKommentar(reasonN.toString());
        } else {
            address.setKommentar(value);
        }
        return address;
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#canHandle(java.lang.String)
     */
    @Override
    public boolean canHandle(final String qrCode) {
        return qrCode.toLowerCase(Locale.getDefault()).startsWith(BezahlcodeContactV2Handler.HANDLER_ID_1)
                || qrCode.toLowerCase(Locale.getDefault()).startsWith(BezahlcodeContactV2Handler.HANDLER_ID_2);
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#getHandlerName()
     */
    @Override
    public String getHandlerName() {
        return BezahlcodeContactV2Handler.HANDLER_NAME;
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#isForBanking()
     */
    @Override
    public boolean isUeberweisung() {
        return false;
    }
}
