/*******************************************************************************
 * Copyright 2017 Frank Mnster
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrcodehandler.bezahlcode;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.rmi.RemoteException;
import java.util.Date;
import java.util.Locale;
import java.util.Map;

import de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerException;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerRegsistry;
import de.frankmuenster.jameica.qrscanner.Settings;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.hbci.rmi.Ueberweisung;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Implementiert das Interface {@link de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler} mit einem Hanlder fr
 * Bezahlcode berweisungen (nicht SEPA)
 *
 * @author Frank Mnster
 */
public class BezahlcodeSinglePaymentHandler implements IQrCodeHandler {

    /**
     * Name des Handlers
     */
    private static final String HANDLER_NAME = "BezahlCode - Authority:singelpayment";

    /**
     * Hander ID
     */
    private static final String HANDLER_ID = "bank://singlepayment?";
    /**
     * Handler registrieren
     */
    static {
        QrCodeHandlerRegsistry.register(new BezahlcodeSinglePaymentHandler());
    }

    /**
     * Standardkonstruktor
     */
    public BezahlcodeSinglePaymentHandler() {
        // Absichtlich leer
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#handleQrCode(java.lang.String)
     */
    @Override
    public void handleQrCode(final String qrCode) throws QrCodeHandlerException {
        if (!qrCode.toLowerCase(Locale.getDefault()).startsWith(BezahlcodeSinglePaymentHandler.HANDLER_ID)) {
            Logger.error("Falscher Qr-Code. Beginnt nicht mit: '" + BezahlcodeSinglePaymentHandler.HANDLER_ID + "'");
            Logger.debug("Qr-Code: " + qrCode);
            throw new QrCodeHandlerException("Qr-Code not valid");
        }
        if (Settings.isHibiscusInstalled()) {
            try {
                final String qrCodeDecode = URLDecoder.decode(qrCode, "ISO-8859-1");
                final String qrCodeQuery = qrCodeDecode.substring(qrCodeDecode.indexOf('?') + 1);
                final Map<String, String> params = BezahlCodeCommenFunctions.getQueryMap(qrCodeQuery);
                final Ueberweisung sepa = this.getSEPA(params);
                if (sepa != null) {
                    // View neu zeichnen
                    GUI.startView(de.willuhn.jameica.hbci.gui.views.UeberweisungNew.class, sepa);
                }
            } catch (final UnsupportedEncodingException e1) {
                throw new QrCodeHandlerException("QR-Code konnte nicht valide dekodiert werden.", e1);
            } catch (final RemoteException e) {
                throw new QrCodeHandlerException("Hibiscus berweisungsobject konnte nicht erstellt werden", e);
            }
        } else {
            final String text = "Der QR-Code enthlt eine berweisung, aber Hibiscus ist nicht installiert.\n"
                    + "Bitte installieren sie das Hibiscus-Plugin.";
            Logger.warn(text);
            try {
                Application.getCallback().notifyUser(text);
            } catch (final Exception e) { // NOPMD
                Logger.error("Info-Fenster konnte nicht geffnet werden. Fehler:\n" + e.getMessage());
            }
        }
    }

    /**
     * @param params
     * @return
     * @throws RemoteException
     */
    @SuppressWarnings({ "PMD.NPathComplexity", "PMD.ModifiedCyclomaticComplexity", "PMD.CyclomaticComplexity" })
    private Ueberweisung getSEPA(final Map<String, String> params) throws RemoteException {
        final Ueberweisung sepa = (Ueberweisung) de.willuhn.jameica.hbci.Settings.getDBService()
                .createObject(Ueberweisung.class, null);
        // Name
        String value;
        if ((value = params.get("name")) != null) {
            if (value.equals(value.toUpperCase(Locale.getDefault()))) {
                try {
                    if (Application.getCallback().askUser(BezahlCodeCommenFunctions.QUESTION_TEXT,
                            new String[] { value })) {
                        value = BezahlCodeCommenFunctions.formatString(value);
                    }
                } catch (final Exception e) { // NOPMD
                    Logger.error("Benutzerabfrage konnte nicht geffnet werden. Fehler:\n" + e.getMessage());
                }
            }
            sepa.setGegenkontoName(value);
        }
        if ((value = params.get("account")) != null) {
            sepa.setGegenkontoNummer(value);
        }
        if ((value = params.get("bnc")) != null) {
            sepa.setGegenkontoBLZ(value);
        }
        if ((value = params.get("amount")) != null) {
            sepa.setBetrag(Double.valueOf(value.replace(",", ".")));
        }
        if ((value = params.get("reason")) == null) {
            final StringBuilder zweck = new StringBuilder();
            if ((value = params.get("reason1")) != null) {
                zweck.append(value).append(' ');
            }
            if ((value = params.get("reason2")) != null) {
                zweck.append(value).append(' ');
            }
            if ((value = params.get("reason3")) != null) {
                zweck.append(value).append(' ');
            }
            if ((value = params.get("reason4")) != null) {
                zweck.append(value);
            }
            sepa.setZweck(zweck.toString());
        } else {
            sepa.setZweck(value);
        }
        if ((value = params.get("executiondate")) != null) {
            final Date termin = new Date(Long.valueOf(value));
            sepa.setTermin(termin);
        }
        return sepa;
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#canHandle(java.lang.String)
     */
    @Override
    public boolean canHandle(final String qrCode) {
        return qrCode.toLowerCase(Locale.getDefault()).startsWith(BezahlcodeSinglePaymentHandler.HANDLER_ID);
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#getHandlerName()
     */
    @Override
    public String getHandlerName() {
        return BezahlcodeSinglePaymentHandler.HANDLER_NAME;
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#isForBanking()
     */
    @Override
    public boolean isUeberweisung() {
        return true;
    }
}
