/*******************************************************************************
 * Copyright 2017 Frank Mnster
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrcodehandler.bezahlcode;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.rmi.RemoteException;
import java.util.Date;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerException;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerRegsistry;
import de.frankmuenster.jameica.qrscanner.Settings;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Implementiert das Interface {@link de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler} mit einem Hanlder fr
 * Bezahlcode SEAP berweisungen
 *
 * @author Frank Mnster
 */
public class BezahlcodeSinglePaymentSepaHandler implements IQrCodeHandler {

    /**
     * Name des Handlers
     */
    private static final String HANDLER_NAME = "BezahlCode - Authority:singelpaymentsepa";

    /**
     * Handler Identifikatiosnstring
     */
    private static final String HANDLER_ID = "bank://singlepaymentsepa?";
    static {
        QrCodeHandlerRegsistry.register(new BezahlcodeSinglePaymentSepaHandler());
    }

    /**
     * Standardkonstruktor
     */
    public BezahlcodeSinglePaymentSepaHandler() {
        // Absichtlich leer
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#handleQrCode(java.lang.String)
     */
    @Override
    public void handleQrCode(final String qrCode) throws QrCodeHandlerException {
        if (!qrCode.toLowerCase(Locale.getDefault()).startsWith(BezahlcodeSinglePaymentSepaHandler.HANDLER_ID)) {
            Logger.error(
                    "Falscher Qr-Code. Beginnt nicht mit: '" + BezahlcodeSinglePaymentSepaHandler.HANDLER_ID + "'"); //$NON-NLS-1$ //$NON-NLS-2$
            Logger.debug("Qr-Code: " + qrCode); //$NON-NLS-1$
            throw new QrCodeHandlerException("Qr-Code not valid"); //$NON-NLS-1$
        }
        if (Settings.isHibiscusInstalled()) {
            try {
                final StringBuffer notifyText = new StringBuffer();
                final String qrCodeDecode = URLDecoder.decode(qrCode, "ISO-8859-1"); //$NON-NLS-1$
                final String qrCodeQuery = qrCodeDecode.substring(qrCodeDecode.indexOf('?') + 1);
                final Map<String, String> params = BezahlCodeCommenFunctions.getQueryMap(qrCodeQuery);
                final AuslandsUeberweisung transaction = this.getSEPA(params, notifyText);
                if (transaction != null) {
                    // View neu zeichnen
                    GUI.startView(de.willuhn.jameica.hbci.gui.views.AuslandsUeberweisungNew.class, transaction);
                }
                if (notifyText.length() > 0) {
                    notifyText.deleteCharAt(notifyText.length() - 1);
                    notifyText.deleteCharAt(notifyText.length() - 1);
                    Application.getMessagingFactory()
                            .sendMessage(new StatusBarMessage(
                                    "Der QR-Code enthlt folgende Probleme:" + notifyText.toString(),
                                    StatusBarMessage.TYPE_ERROR));
                }
            } catch (final UnsupportedEncodingException e1) {
                throw new QrCodeHandlerException("QR-Code konnte nicht valide dekodiert werden.", e1);
            } catch (final RemoteException e) {
                throw new QrCodeHandlerException("Hibiscus Auslandsberweisungsobject konnte nicht erstellt werden.",
                        e);
            }
        } else {
            final String text = "Der QR-Code enthlt eine SEPA-berweisung, aber Hibiscus ist nicht installiert.\n"
                    + "Bitte installieren sie das Hibiscus-Plugin.";
            Logger.warn(text);
            try {
                Application.getCallback().notifyUser(text);
            } catch (final Exception e) { // NOPMD
                Logger.error("Info-Fenster konnte nicht geffnet werden. Fehler:\n" + e.getMessage());
            }
        }
    }

    /**
     * @param params
     *            die einzelnen Segmente der URL
     * @param notifyText
     *            Buffer for Infotext
     * @return Die Auslandsberweisung oder NULL
     * @throws RemoteException
     */
    @SuppressWarnings({ "PMD.NPathComplexity", "PMD.ModifiedCyclomaticComplexity", "PMD.CyclomaticComplexity" })
    private AuslandsUeberweisung getSEPA(final Map<String, String> params, final StringBuffer notifyText)
            throws RemoteException {
        AuslandsUeberweisung sepaUeberweisung = null;
        sepaUeberweisung = (AuslandsUeberweisung) de.willuhn.jameica.hbci.Settings.getDBService()
                .createObject(AuslandsUeberweisung.class, null);
        // Name
        String value;
        if ((value = params.get("name")) != null) {
            if (value.length() > 0 && value.equals(value.toUpperCase(Locale.getDefault()))) {
                try {
                    if (Application.getCallback().askUser(BezahlCodeCommenFunctions.QUESTION_TEXT,
                            new String[] { value })) {
                        value = BezahlCodeCommenFunctions.formatString(value);
                    }
                } catch (final Exception e) { // NOPMD
                    Logger.error("Benutzeranfrage fehlgeschlagen.", e);
                }
            }
            sepaUeberweisung.setGegenkontoName(value);
        }
        value = params.get("iban");
        if (StringUtils.isEmpty(value)) {
            notifyText.append(" IBAN leer |");
        } else {
            sepaUeberweisung.setGegenkontoNummer(value);
        }
        value = params.get("bic");
        if (StringUtils.isNotEmpty(value)) {
            sepaUeberweisung.setGegenkontoBLZ(value);
        }
        value = params.get("amount");
        if (StringUtils.isNotEmpty(value)) {
            try {
                sepaUeberweisung.setBetrag(Double.valueOf(value.replace(",", ".")));
            } catch (final NumberFormatException nfe) {
                notifyText.append(" Betrag ungltig |");
            }
        }
        sepaUeberweisung.setZweck(StringUtils.EMPTY);
        if ((value = params.get("reason")) == null) {
            final StringBuilder zweck = new StringBuilder();
            if ((value = params.get("reason1")) != null) {
                zweck.append(value).append(' ');
            }
            if ((value = params.get("reason2")) != null) {
                zweck.append(value).append(' ');
            }
            if ((value = params.get("reason3")) != null) {
                zweck.append(value).append(' ');
            }
            if ((value = params.get("reason4")) != null) {
                zweck.append(value);
            }
            sepaUeberweisung.setZweck(zweck.toString());
        } else {
            sepaUeberweisung.setZweck(value);
        }
        if (sepaUeberweisung.getZweck().equals("")) {
            notifyText.append(" Verwendungszweck leer |");
        }
        if ((value = params.get("separeference")) != null) {
            sepaUeberweisung.setEndtoEndId(value);
        }
        if ((value = params.get("executiondate")) != null) {
            final Date termin = new Date(Long.valueOf(value));
            sepaUeberweisung.setTermin(termin);
        }
        return sepaUeberweisung;
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#canHandle(java.lang.String)
     */
    @Override
    public boolean canHandle(final String qrCode) {
        return qrCode.toLowerCase(Locale.getDefault()).startsWith(BezahlcodeSinglePaymentSepaHandler.HANDLER_ID);
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#getHandlerName()
     */
    @Override
    public String getHandlerName() {
        return BezahlcodeSinglePaymentSepaHandler.HANDLER_NAME;
    }

    /**
     * @see de.frankmuenster.jameica.qrcodehandler.IQrCodeHandler#isForBanking()
     */
    @Override
    public boolean isUeberweisung() {
        return true;
    }
}
