/*******************************************************************************
 * Copyright 2017 Frank Mnster
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.qrscanner.gui.parts;

import java.rmi.RemoteException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

import de.willuhn.jameica.gui.Part;
import de.willuhn.logging.Logger;

/**
 * Erstellt ein ImageLable basierend auf einem SWT Label, das zur Darstellung
 * von Text und Bilder benutzt werden kann. Die Gre wird entweder durch ein
 * Bild oder ein Grenangabe im Konstruktor gesetzt und ist dann nicht mehr
 * vernderbar.
 *
 * @author Frank Mnster
 */
public class ImageLabelPart implements Part, PaintListener {

	/**
	 * Image das angezeigt werden soll
	 */
	private transient Image img;

	/**
	 * Text, der angezeigt werden soll
	 */
	private transient String text;

	/**
	 * Gre des Lables
	 */
	private transient Point size;

	/**
	 * Das Label zum anzeigen des Images/Textes
	 */
	private transient Label lbl;

	/**
	 * Erstellt das ImageLable ohne weitere Inhalte
	 */
	public ImageLabelPart() {
		// absichtlich leer
	}

	/**
	 * Erstellt eine ImageLabel mit Image und setzt die Gre des Labels auf die
	 * Bildgre
	 *
	 * @param img
	 *            Image das dargestellt werden soll.
	 */
	public ImageLabelPart(final Image img) {
		this.img = img;
	}

	/**
	 * Erstellt eine ImageLabel mit leeren Inhalt in der angegebenen Gre.
	 *
	 * @param size
	 *            Point-Object mit Grenangaben
	 */
	public ImageLabelPart(final Point size) {
		this.size = size;
	}

	/**
	 * Erstellt eine ImageLabel mit Bild in der angegebenen Gre.
	 *
	 * @param img
	 *            Image das dargestellt werden soll.
	 * @param size
	 *            Point-Object mit Grenangaben
	 */
	public ImageLabelPart(final Image img, final Point size) {
		this.img = img;
		this.size = size;
	}

	/**
	 * Setzt ein neues Image und zeigt es an
	 *
	 * @param img
	 *            IMage, dass angezeigt werden soll
	 */
	public void setImage(final Image img) {
		this.img = img;
		// draw image if lbl exists
		if (this.lbl != null) {
			Display.getDefault().asyncExec(new Runnable() {

				/**
				 * @see Runnable#run()
				 */
				@Override
				public void run() {
					// ... do any work that updates the screen ...
					if (!ImageLabelPart.this.lbl.isDisposed()) {
						ImageLabelPart.this.lbl.setImage(img);
						ImageLabelPart.this.lbl.redraw();
						ImageLabelPart.this.lbl.update();
						ImageLabelPart.this.lbl.getParent().layout();
					}
				}
			});
		}
	}

	/**
	 * Setzt einen neuen Text, der angezeigt werden soll. Der Text ersetzt auch
	 * ein evtl. angezeigtes Image.
	 *
	 * @param text
	 *            der angezeigt werden soll
	 */
	public void setText(final String text) {
		this.text = text;
		// draw image if lbl exists
		if (this.lbl != null) {
			Display.getDefault().asyncExec(new Runnable() {

				/**
				 * @see Runnable#run()
				 */
				@Override
				public void run() {
					// ... do any work that updates the screen ...
					if (!ImageLabelPart.this.lbl.isDisposed()) {
						ImageLabelPart.this.lbl.setText(text);
						ImageLabelPart.this.lbl.redraw();
						ImageLabelPart.this.lbl.update();
						ImageLabelPart.this.lbl.getParent().layout();
					}
				}
			});
		}
	}

	/**
	 * Liefert den aktuellen Text
	 */
	public String getText() {
		return this.text;
	}

	/**
	 * Gibt die Gre des ImageLabels als Point zurck
	 *
	 * @return Gre des ImageLables
	 */
	public Point getSize() {
		return this.size;
	}

	/**
	 * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void paint(final Composite parent) throws RemoteException {
		// final GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
		// | GridData.FILL_VERTICAL);
		if (parent == null) {
			Logger.error("Parent is null");
		}
		final Composite container = new Composite(parent, SWT.BORDER);
		this.lbl = new Label(container, SWT.WRAP);
		if (this.getSize() != null) {
			this.lbl.setSize(this.getSize());
		}
		if (this.img != null) {
			this.lbl.setImage(this.img);
		}
		final GridData VAlignedGD = new GridData();
		VAlignedGD.verticalAlignment = SWT.CENTER;
		VAlignedGD.grabExcessVerticalSpace = true;
		VAlignedGD.horizontalAlignment = SWT.CENTER;
		VAlignedGD.grabExcessHorizontalSpace = true;
		this.lbl.setLayoutData(VAlignedGD);
	}

	/**
	 * @see Part#paint(Composite)
	 */
	@Override
	public void paintControl(final PaintEvent event) {
		Logger.debug(this.getClass().getName() + " - paintControl");
		if (this.img != null) {
			event.gc.drawImage(this.img, 0, 0);
		}
	}
}
