/*******************************************************************************
 * Copyright 2017 Frank Mnster
 *  
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.qrscanner.ipcam;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import com.github.sarxos.webcam.ds.ipcam.IpCamDevice;
import com.github.sarxos.webcam.ds.ipcam.IpCamMode;

import de.willuhn.logging.Logger;

/**
 * Spezielle IpCamDevice Implementierung f&uuml;r DroidCam App. &Uuml;berschreibt isOnline und getImage.
 *
 * @author Frank Mnster
 * @version 1.0
 */
public class DroidCamDevice extends IpCamDevice {

    /**
     * Zeitspanne zwischen zwei Auto-Fokus aufrufen.
     */
    private static final int FOCUS_INTERVALL = 2000;

    /**
     * HTTP-timeout f&uuml;r Auto-Fokus Anfrage
     */
    private final static int TIMEOUT_AUTOFOCUS = 200;

    /**
     * HTTP-timeout f&uuml;r Ping Anfrage
     */
    private final static int TIMEOUT_PING = 500;

    /**
     * Der Host des Streams. Kann auch IP-Adresse sein.
     */
    private transient String host;

    /**
     * Der Port des Streams.
     */
    private transient int port;

    /**
     * Zeitstempel des letzten Auto-Fokus-Aufrufs.
     */
    private transient long camLastReadTime;

    /**
     * Erstellt ein DroidCam Instance durch Angabe der IP und des Ports.
     *
     * @param host
     *            Der Host des Streams. Kann auch IP-Adresse sein.
     * @param port
     *            Der Port des Streams.
     */
    public DroidCamDevice(final String host, final int port) {
        this(DroidCamDevice.getCamUrl(host, port));
    }

    /**
     * Erstellt ein DroidCam Instance durch Angabe der URL.
     *
     * @param url
     *            Die URL zu dem mjpegfeed Stream der DroidCam App. Das Suffix mjpegfeed muss mit angegeben werden.
     */
    public DroidCamDevice(final URL url) {
        super("DroidCam", url, IpCamMode.PUSH);
        this.host = url.getHost();
        this.port = url.getPort();
        // internal parameter
        this.camLastReadTime = 0;
    }

    /**
     * Liest ein Bild von der DroidCam. Sendet alle 2 Sekunden eine Auto-Fokus Anfrage.
     */
    @Override
    public BufferedImage getImage() {
        final long now = System.currentTimeMillis();
        if (now - this.camLastReadTime > DroidCamDevice.FOCUS_INTERVALL) {
            this.focusCam();
            this.camLastReadTime = System.currentTimeMillis();
        }
        BufferedImage image;
        synchronized (this) {
            image = super.getImage();
        }
        return image;
    }

    /**
     * Erstellt ein URL aus angegeben Host und Port.
     *
     * @param host
     *            Der Host des Streams. Kann auch IP-Adresse sein.
     * @param port
     *            Der Port des Streams.
     * @return Ein URL Objekt, mit Host und Port, das auf den DriodCam mjpegfeed Stream zeigt.
     */
    private static URL getCamUrl(final String host, final int port) {
        URL url = null;
        try {
            url = new URL("http", host, port, "/mjpegfeed");
            Logger.debug("DroidCam URL: " + url.toString());
        } catch (final MalformedURLException e) {
            Logger.error("DroidCam URL falsch.");
        }
        return url;
    }

    /**
     * Sendet einen Auto-Fokus Reqest and die DriodCam
     */
    private void focusCam() {
        try {
            final URL url = new URL("http", this.host, this.port, "/cam/1/af");
            final HttpURLConnection connection = (HttpURLConnection) url.openConnection();
            connection.setConnectTimeout(DroidCamDevice.TIMEOUT_AUTOFOCUS);
            connection.setReadTimeout(DroidCamDevice.TIMEOUT_AUTOFOCUS);
            final int responseCode = connection.getResponseCode();
            connection.disconnect();
            Logger.debug("Fokusrequest Antwort: " + responseCode);
        } catch (final IOException exception) {
            Logger.debug("Auto-Fokus request konnte nicht gesendet werden. Fehler: " + exception.getMessage());
        }
    }

    /**
     * Pings the DroidCam URL. This effectively sends a GET request and returns <code>true</code> if the response code
     * is in the 200-399 range.
     *
     * @return <code>true</code> if the given HTTP URL has returned response code 200-399 on a HEAD request within the
     *         given TIMEOUT_PING, otherwise <code>false</code>.
     */
    @Override
    public boolean isOnline() {
        boolean isOnline;
        try {
            final URL url = new URL("http", this.host, this.port, "");
            final HttpURLConnection connection = (HttpURLConnection) url.openConnection();
            connection.setConnectTimeout(DroidCamDevice.TIMEOUT_PING);
            connection.setReadTimeout(DroidCamDevice.TIMEOUT_PING);
            // connection.setRequestMethod("HEAD");
            final int responseCode = connection.getResponseCode();
            connection.disconnect();
            isOnline = 200 <= responseCode && responseCode <= 399;
        } catch (final IOException exception) {
            isOnline = false;
        }
        return isOnline;
    }
}
